/**
 *    @file  main.c
 *   @brief
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  25/10/2024
 *      Revision:  none
 * Last modified:  2024-11-10 15:25
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ansi-color-codes.h"
#include "revision.h"
#include "csv.h"                                /* CSV manipulation functions */
#include "clusterlst.h"                         /* Cluster list structure */
#ifndef NOCPPCHECK
#include "rsstats-opts.h"                       /* Libopts generated options */
#endif
#include "rptbdbs.h"
#include "rptcluster.h"

#include <stdlib.h>
#include <string.h>

int main (int argc, char** argv, char** env) {
    FILE *reportfile;
    FILE *configfile;
    char  configline[1024];
    cluster_t* cluster;

    (void)env;                                  /* Avoid unused warning/error */
    printf(BCYN PACKAGE_NAME " " PACKAGE_VERSION RESET "\n");
#ifdef REVISION
    printf("Revision " BBLU REVISION RESET);
#endif
#ifdef BBID
    printf(" Build #" BBID);
#endif
    printf("\n");

#pragma GCC diagnostic push                             /* save the actual diag context */
#pragma GCC diagnostic ignored "-Wdate-time"         /* locally disable warnings because of non reproductible build triggered by pbuild */
    printf("Compiled %s at %s\n",__DATE__, __TIME__);
#pragma GCC diagnostic pop                              /* restore previous diag context */
    printf("Copyright 2024 François Cerbelle\n");
    printf("Report bugs to %s\n\n", BYEL PACKAGE_BUGREPORT RESET);
    /* AutoGen option parsing and consumming */
    {
        int arg_ct = optionProcess( &rsstatsOptions, argc, argv );
        argc -= arg_ct;
        argv += arg_ct;
    }

    printf("==> Read input file (cluster definitions): %s\n",OPT_ARG(INPUT));
    /* Open the cluster definitions files */
    configfile = fopen(OPT_ARG(INPUT), "r");
    if (!configfile) {
        perror("main open(configfile)");
        exit(EXIT_FAILURE);
    }

    unsigned int lineno = 0;
    while (fgets(configline, sizeof(configline), configfile) != NULL) {
        lineno++;

        /* Allocate a cluster record */
        if(NULL==(cluster=malloc(sizeof(struct cluster_s)))) {
            perror("main malloc(cluster)");
            exit(EXIT_FAILURE);
        }

        /* Extract cluster information from the configline */
        if (NULL==(cluster->host = csvtok(configline))) {
            fprintf(stderr,"Bad configline (%s:%ud)\n", OPT_ARG(INPUT), lineno);
            free(cluster);
            continue;
        }
        if (NULL==(cluster->user = csvtok(NULL))) {
            fprintf(stderr,"Bad configline (%s:%ud)\n", OPT_ARG(INPUT), lineno);
            free(cluster->host);
            free(cluster);
            continue;
        }
        if (NULL==(cluster->pass = csvtok(NULL))) {
            fprintf(stderr,"Bad configline (%s:%ud)\n", OPT_ARG(INPUT), lineno);
            free(cluster->host);
            free(cluster->user);
            free(cluster);
            continue;
        }
        if (NULL==(cluster->insecure = csvtok(NULL))) {
            fprintf(stderr,"Bad configline (%s:%ud)\n", OPT_ARG(INPUT), lineno);
            free(cluster->host);
            free(cluster->user);
            free(cluster->pass);
            free(cluster);
            continue;
        }
        if (NULL==(cluster->cacert = csvtok(NULL))) {
            fprintf(stderr,"Bad configline (%s:%ud)\n", OPT_ARG(INPUT), lineno);
            free(cluster->host);
            free(cluster->user);
            free(cluster->pass);
            free(cluster->insecure);
            free(cluster);
            continue;
        }

        /* Check if the configline should be processed */
        if ((NULL!=strstr(OPT_ARG(CLUSTERS),"all"))
                ||(NULL!=strstr(OPT_ARG(CLUSTERS),cluster->host)))
            cluster->enabled=1;
        else
            cluster->enabled=0;
        if (clusterlist_find(cluster->host))
            fprintf(stderr,"Double cluster definition (%s @ %s:%ud)\n",
                    cluster->host, OPT_ARG(INPUT), lineno);
        clusterlist_add(cluster);
    }
    fclose(configfile);

    printf("==> Clusters to query : ");
    cluster=clusterlist_first();
    while(cluster) {
        if (cluster->enabled==1)
            printf("%s ",cluster->host);
        cluster = clusterlist_next();
    }
    printf("\n");

    printf("==> Open output file (report): %s\n",OPT_ARG(OUTPUT));

    /* Open the output file */
    reportfile = fopen(OPT_ARG(OUTPUT),"w");
    if (!reportfile) {
        perror("Opening output file");
        exit(EXIT_FAILURE);
    }

    /* Execute cluster report ? all or cluster specified */
    if (OPT_VALUE_REPORTS & REPORTS_CLUSTER) {
        printf("==> Running reports (cluster)\n");
        fprintf(reportfile,"\nclusters:\n");
        report_cluster_header(reportfile);
        /* Execute report against enabled clusters */
        cluster=clusterlist_first();
        while(cluster) {
            /* Check if the configline should be processed */
            if (cluster->enabled==1) {
                printf("==> Running report (clusters) on cluster %s\n",cluster->host);
                report_cluster(reportfile, cluster);
            }
            cluster = clusterlist_next();
        }
    }

    /* Execute bdbs report ? all or bdbs specified */
    if (OPT_VALUE_REPORTS & REPORTS_BDBS) {
        printf("==> Running reports (bdbs)\n");
        fprintf(reportfile,"\nbdbs:\n");
        report_bdbs_header(reportfile);
        /* Execute report against enabled clusters */
        cluster=clusterlist_first();
        while(cluster) {
            /* Check if the configline should be processed */
            if (cluster->enabled==1) {
                printf("==> Running report (bdbs) on cluster %s\n",cluster->host);
                report_bdbs(reportfile, cluster);
            }
            cluster = clusterlist_next();
        }
    }

    fclose(reportfile);

#ifdef _WIN32
    system("PAUSE");  /* For windows console window to wait. */
#endif

    return EXIT_SUCCESS;
}
/* vim: set tw=80: */
