/**   @file  suite_csvtranscode.c
 *   @brief  Check testsuite template
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-10 17:32
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include "checktools.inc"

#include "csv.c"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(csvtranscode_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

int txt2csvtest(const char* txt, const char* exp) {
    char* src;
    char* dst;
    int retval;
    src=strdup(txt);
    dst = txt2csv(src);
    retval=strcmp(dst,exp);
    free(src);
    free(dst);
    return retval;
}

START_TEST(csvtranscode_txt2csv) {

    ck_assert(0==txt2csvtest("",""));
    ck_assert(0==txt2csvtest("ab","ab"));

    ck_assert(0==txt2csvtest(",ab","\",ab\""));
    ck_assert(0==txt2csvtest("a,b","\"a,b\""));
    ck_assert(0==txt2csvtest("ab,","\"ab,\""));

    ck_assert(0==txt2csvtest(" ab"," ab"));
    ck_assert(0==txt2csvtest("a b","a b"));
    ck_assert(0==txt2csvtest("ab ","ab "));

    ck_assert(0==txt2csvtest("\rab","\"\rab\""));
    ck_assert(0==txt2csvtest("a\rb","\"a\rb\""));
    ck_assert(0==txt2csvtest("ab\r","\"ab\r\""));

    ck_assert(0==txt2csvtest("\nab","\"\nab\""));
    ck_assert(0==txt2csvtest("a\nb","\"a\nb\""));
    ck_assert(0==txt2csvtest("ab\n","\"ab\n\""));

    ck_assert(0==txt2csvtest("\r\nab","\"\r\nab\""));
    ck_assert(0==txt2csvtest("a\r\nb","\"a\r\nb\""));
    ck_assert(0==txt2csvtest("ab\r\n","\"ab\r\n\""));

    ck_assert(0==txt2csvtest("\n\rab","\"\n\rab\""));
    ck_assert(0==txt2csvtest("a\n\rb","\"a\n\rb\""));
    ck_assert(0==txt2csvtest("ab\n\r","\"ab\n\r\""));

    ck_assert(0==txt2csvtest("\"ab","\"\"\"ab\""));
    ck_assert(0==txt2csvtest("a\"b","\"a\"\"b\""));
    ck_assert(0==txt2csvtest("ab\"","\"ab\"\"\""));
}
END_TEST

int csvtranscode_csvtok_ok_helper(char* csv, const char* expected) {
    char* txt = csvtok(csv);
    ck_assert(((NULL!=expected)&&(NULL!=txt))||
              ((NULL==expected)&&(NULL==txt))) ;
    int retval = (NULL==txt?0:strcmp(txt,expected));
    free(txt);
    return retval;
}

START_TEST(csvtranscode_csvtok_ok) {
    char* csv = strdup(
                    " ,a, b,c ,\"\"\"\",\" ,a, b\",c , ,a, b,\"c ,\",\",a, b,c ,\",\"a\rb\",\"a\nb\",\"a\r\nb\",\"a\n\rb\", \r\n"
                    " ,a, b,c ,\"\"\"\",\" ,a, b\",c , ,a, b,\"c ,\",\",a, b,c ,\",\"a\rb\",\"a\nb\",\"a\r\nb\",\"a\n\rb\", ");

    ck_assert(0==csvtranscode_csvtok_ok_helper(csv," "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"\""));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," ,a, b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c ,"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,",a, b,c ,"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\rb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\nb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\r\nb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\n\rb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," "));

    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"\""));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," ,a, b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," "));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," b"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"c ,"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,",a, b,c ,"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\rb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\nb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\r\nb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,"a\n\rb"));
    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL," "));

    ck_assert(0==csvtranscode_csvtok_ok_helper(NULL,NULL));

    free(csv);
}
END_TEST

START_TEST(csvtranscode_csvtok_empty) {
    char* csv = strdup("");
    char* txt = NULL;

    txt = csvtok(csv);
    ck_assert(NULL==txt);
    free(txt);
    free(csv);
}
END_TEST

START_TEST(csvtranscode_csvtok_fail) {
    /*
    FAILURE expected
    a,
    a,\r
    a,\n
    a,\r\n
    a,\n\r
    a, "b",c
    a,"b" ,c
    a,"b,c
    */

    ck_assert(0); /* To Be Implemented */
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void csvtranscode_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void csvtranscode_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void csvtranscode_unchecked_common_setup() {
    forktest_init();
}
void csvtranscode_unchecked_common_teardown() {
}
/************************************************************************/
Suite* csvtranscode_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("CSVTranscode");

    tc = tcase_create("csvtok");
    tcase_set_tags(tc,"CSVTranscodeTag CSVTranscodeTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, csvtranscode_checked_uninitialized_setup, csvtranscode_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, csvtranscode_unchecked_common_setup, csvtranscode_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, csvtranscode_test,6);
    tcase_add_test(tc, csvtranscode_txt2csv);
    tcase_add_test(tc, csvtranscode_csvtok_empty);
    tcase_add_test(tc, csvtranscode_csvtok_ok);
    tcase_add_test(tc, csvtranscode_csvtok_fail);

    return s;
}
/* vim: set tw=80: */
